#!/usr/local/bin/perl
# E:/perl/bin/perl

use lib "E:/perl/site/lib";
use Tk;

use strict;

# Main window
# Need this here as some of the initialisation uses $mw
my $mw = MainWindow->new;


###  Initialise various things
#

# Get the slide show directory, load the presentation name, slide names,
# get the slide titles and populate the option menu
my($slidedir,$prestitle,@slides,@titles,@options) = ();
getSlides();


# Fonts and colours for menubar
my(@families,@sizes) = ();
my(@fgcols,@bgcols) = ();
getFontsColours();

# Set inital current fonts and colours
my($family,$size,$weight) = ("Arial",24,"normal");
my($bgcol,$fgcol) = ("grey","black");

# Current slide, its title and text
my $cslide = 0;
my($cslide,$title,$text) = (0,$titles[0],"Loading ...");

#
###


# Title
$mw->title($prestitle);

# Menubar
makeMenuBar();


# Slide
my $slide = $mw->Label(
             -font => [ -family => $family,
                        -size => $size,
                        -weight => $weight ],
             -textvariable => \$text
           )->pack(
              -side=>"bottom",
              -fill=> 'both'
              );


my $prev = $mw->Button( 
      -text => "Prev", 
      -underline => 0,
      -cursor => "sb_left_arrow",
      -command => sub{ go(-1) } )->pack(
      -side => "left"
      );
$prev->bind('<KeyPress-P>' => sub{ go(-1) });
$prev->bind('<KeyPress-p>' => sub{ go(-1) });

my $next = $mw->Button( 
      -text => "Next", 
      -underline => 0,
      -cursor => "sb_right_arrow",
      -command => sub{ go(1) } )->pack(
      -side => "left"
      );
$next->bind('<KeyPress-N>' => sub{ go(1) });
$next->bind('<KeyPress-n>' => sub{ go(1) });

my $chosen = $mw->Optionmenu(
             -options => \@options,
             -textvariable => \$title,
             -variable => \$cslide,
             -cursor => "sb_h_double_arrow",
             -font => [-size => $size],
             -command => sub{ go() }
            )->pack(
          -side => "left",
          -before=>$next,
          -expand => 1
          );



=cut
# During development
$mw->Button( -text => "Open Me Again", -command => sub { 
              system("e:/perl/bin/perl e:/projects/perltk/pres.pl") } )->pack(
      -side => "top" );
=cut


# Small/Large via Mouse
$mw->bind('<Button-3>' => sub { $weight="bold";  $size = 12; setFont() });
$mw->bind('<Double-Button-3>' => sub { $size = 24; $weight="normal"; setFont() });

# Set the current and then show
go();
setColours();

MainLoop;


print "Back to here\n";


sub getSlides {
  $slidedir = "/home/grant/tk";
  open(I,"<$slidedir/index.txt") || die $!;
  $prestitle = <I>;
  chomp $prestitle;
  my($slide,$title) = ();
  while($slide = <I>) {
    chomp $slide;
    push @slides, $slide;
    if( open(S,"<$slidedir/$slide") ) {
      $title = <S>;
      chomp $title;
      push @titles, $title;
      close S;
    }
    else {
      push @titles, "Missing slide $slide";
    }
  }
  close I;
  for(my $i=0;$i<=$#titles;$i++) {
    $options[$i] = [ $titles[$i],$i ];
  }
}
    
    
sub getFontsColours {

  for my $fam (sort $mw->fontFamilies) {
    push @families,
    ['radiobutton', $fam, -variable => \$family, -command => \&setFont ];
  }

  for my $tmp (1 .. 24) {
    my $siz = 8 + 4*$tmp;
    push @sizes,
    ['radiobutton', $siz, -variable => \$size, -command => \&setFont ];
  }

  for my $col ("black", "blue", "grey", "red", "yellow", "white") {
    push @fgcols,
    ['radiobutton', $col, -variable => \$fgcol, -command => \&setColours ];
    push @bgcols,
    ['radiobutton', $col, -variable => \$bgcol, -command => \&setColours ];
  }

}


sub makeMenuBar {

  $mw->configure(-menu => my $menubar = $mw->Menu);

  my $msize = $menubar->cascade(
     -label => '~Font', -menuitems => 
       [
         [ "cascade", "~Family", -menuitems => \@families ],
         [ "cascade", "~Size", -menuitems => \@sizes ],
         [ "cascade", "~Weight", -menuitems =>
           [
              [ "checkbutton", "~Bold", 
                  -offvalue=>"normal",
                  -onvalue=>"bold",
                  -variable=>\$weight,
                  -command => \&setFont ]
           ]
         ]
       ]
     );

  my $mcol = $menubar->cascade(
     -label => '~Colours', -menuitems => 
       [
         [ "cascade", "~Background", -menuitems => \@bgcols ],
         [ "cascade", "~Foregraound", -menuitems => \@fgcols ],
       ]
     );
}


sub go {

  # Called when Prev and Next are clicked and Options change

  # Prev and Next
  if($_[0] == -1 && $cslide != 0) {
    $cslide--;
  }
  elsif($_[0] == 1 && $cslide != $#titles) {
    $cslide++;
  }

  # Set button status - even if not changed
  if($cslide == 0) {
    $prev->configure(-state=>'disabled');
    $next->configure(-state=>'active');
  }
  elsif($cslide == $#titles) {
    $prev->configure(-state=>'active');
    $next->configure(-state=>'disabled');
  }
  else {
    $prev->configure(-state=>'active');
    $next->configure(-state=>'active');
  }

  # Set Options - really only needed on Prev/Next
  $title = $titles[$cslide];

  # Load the slide
  my $file = $slidedir . "/" . $slides[$cslide];
  if( !open(O,"<$file") ) {
    $text = "Whoa, where'd the file go?";
  }
  else {
    my $tmp = <O>;   #  skip title
    my @text = <O>;
    $text = join "", @text;
  }
  close O;

  # Centre first slide
  if($cslide == 0) {
    $slide->configure(-justify=>"center");
  }
  else {
    $slide->configure(-justify=>"left");
  }

  # Dynamic Outline
  if($cslide == 1) {
    $text = "";
    for(my $i=2;$i<=$#titles;$i++) {
      $text .= "* $titles[$i]\n";
    }
  }
  
}


sub setFont {
  # Font changes are applied to title and slide

  $chosen->configure(
             -font => [ -family=>$family,
                        -size=>$size,
                        -weight=>$weight ]
         );

  $slide->configure(
             -font => [ -family=>$family,
                        -size=>$size,
                        -weight=>$weight ]
         );
}


sub setColours {
  # Colours are applied to everything
  $mw->setPalette(background => $bgcol, foreground => $fgcol );

  # Applied to title
  $chosen->configure(-background => $bgcol, -foreground => $fgcol );

  # Applied to slide
  $slide->configure(-background => $bgcol, -foreground => $fgcol );

  # Applied to previous and next
  $prev->configure(-background => $bgcol, -foreground => $fgcol );
  $next->configure(-background => $bgcol, -foreground => $fgcol );

}


######################

=cut
$mw->Optionmenu(
#         -options => ['Arial', 'Courier'],
         -options => [ sort $mw->fontFamilies ],
         -textvariable => \$family,
         -command => \&setFont
            )->pack(
         -side => "left",
#         -anchor => "n",
         -expand => 1
         );
$mw->Optionmenu(
         -options => [12 .. 96 ],
         -textvariable => \$size,
         -command => \&setFont
            )->pack(
         -side => "left",
#         -anchor => "n",
         -expand => 1
         );
$mw->Checkbutton(-text=>'Bold', 
                 -offvalue=>"normal",
                 -onvalue=>"bold",
                 -variable=>\$weight,
         -command => \&setFont
            )->pack(
         -side => "left",
#         -anchor => "n",
         -expand => 1,
         -fill => 'x'
         );
=cut
